/*  Virtual File System support
 *   - prototypes functions and types
 *  draft version 2
 *
 *  used in Seksi commander and Tux Commander
 *
 *  Copyright (C) 2003 Radek Cervinka <radek.cervinka@centrum.cz>
 *  Copyright (C) 2007 Tomas Bzatek <tbzatek@users.sourceforge.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#define _FILE_OFFSET_BITS 64

#include <stdio.h>
#include <stdint.h>
#include <sys/types.h>

//  using namespace std;

typedef int TVFSResult;


typedef void (* TVFSLogFunc)(char *s);
typedef long (* TVFSCopyCallBackFunc)(u_int64_t iPos, u_int64_t iMax, void *data);
typedef void *TVFSFileDes;

static const int cVFSVersion = 2;     //  current version of the VFS API

//  Capabilities
static const int capVFS_nil = 0;
static const int capVFS_List = 1 << 0;
static const int capVFS_CopyOut = 1 << 1;
static const int capVFS_CopyIn = 1 << 2;
static const int capVFS_MkDir = 1 << 3;
static const int capVFS_RmDir = 1 << 4;
static const int capVFS_Multiple = 1 << 5;  //  support multiple files = background copy & thread safe
static const int capVFS_Delete = 1 << 6;
static const int capVFS_Rename = 1 << 7;
static const int capVFS_Execute = 1 << 8;
static const int capVFS_Append = 1 << 9;

//  Error codes (TVFSResult)
static const int cVFS_OK = 0;
static const int cVFS_Failed = 1;   //  also No such file
static const int cVFS_Cancelled = 2;
static const int cVFS_Not_Supported = 3;
static const int cVFS_No_More_Files = 4;
static const int cVFS_ReadErr = 5;
static const int cVFS_WriteErr = 6;  //  also Readonly FileSystem
static const int cVFS_LoginFailed = 7;
static const int cVFS_PermissionDenied = 8;
static const int cVFS_NoSpaceLeft = 9;
static const int cVFS_mallocFailed = 10;


//  Open modes
enum {
  cVFS_OpenRead,
  cVFS_OpenWrite,
  cVFS_OpenAppend
};


//  Item Type enum
typedef enum {
  vRegular = 0,
  vSymlink = 1,
  vChardev = 2,
  vBlockdev = 3,
  vDirectory = 4,
  vFifo = 5,
  vSock = 6,
  vOther = 7
} TVFSItemType;


typedef struct {
  char *sFileName;
  u_int64_t iSize;
  __time_t m_time;    //  numbers should be located before the other variables (bug?)
  __time_t a_time;
  __time_t c_time;
  __mode_t iMode;
  char *sLinkTo;
  __uid_t iUID;
  __gid_t iGID;
  TVFSItemType ItemType;
} TVFSItem;

typedef struct {
  const char *Name;
  const char *Description;
  const char *About;
  const char *Copyright;
} TVFSInfo;

