/*  Tux Commandern VFS: String utilities
 *  Copyright (C) 2007 Tomas Bzatek <tbzatek@users.sourceforge.net>
 *   Check for updates on tuxcmd.sourceforge.net
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <string>
#include <stdlib.h>
#include "strutils.h"


char* include_trailing_path_sep(const char *APath)
{
  if (APath == NULL) return NULL;
  char *ANewPath;
//  fprintf(stderr, "xxx = %s\n", (APath + strlen(APath) - 1));
  if (strcmp(APath + strlen(APath) - 1, "/") != 0) {
    ANewPath = (char*)malloc(strlen(APath) + 2);
    snprintf(ANewPath, strlen(APath) + 2, "%s/", APath);
  } else ANewPath = strdup(APath);
  return ANewPath;
}

char* exclude_trailing_path_sep(const char *APath)
{
  if (APath == NULL) return NULL;
  char *ANewPath;
//  fprintf(stderr, "orig = %s, len = %i, xxx = %s\n", APath, strlen(APath), (APath + strlen(APath) - 1));
  if ((strcmp(APath + strlen(APath) - 1, "/") == 0) && (strlen(APath) > 1)) {
    ANewPath = (char*)malloc(strlen(APath));
    snprintf(ANewPath, strlen(APath), "%s", APath);  //  The number of bytes copied includes the trailing \0
  } else ANewPath = strdup(APath);
  return ANewPath;
}

char* include_leading_path_sep(const char *APath)
{
  if (APath == NULL) return NULL;
  char *ANewPath;
//  fprintf(stderr, "xxx = %s, %i, strlen(APath) = %i, index() = %i\n", APath, APath, strlen(APath), index(APath, 0x2f));
  if (index(APath, 0x2f) != APath) {
    ANewPath = (char*)malloc(strlen(APath) + 2);
    snprintf(ANewPath, strlen(APath) + 2, "/%s", APath);    //  The number of bytes copied includes the trailing \0
  } else ANewPath = strdup(APath);
  return ANewPath;
}

char* exclude_leading_path_sep(const char* APath)
{
  if (APath == NULL) return NULL;
  char *s = strdup(APath);
  char *ss;
  if (IS_DIR_SEP(*s)) ss = strdup(s + 1); else ss = strdup(s);
  free(s);
  return ss;
}


char* extract_file_name(const char *APath)
{
  if (APath == NULL) return NULL;
//  fprintf(stderr, "xxx = %s\n", (APath + strlen(APath) - 1));
  char *file_part = rindex(APath, 0x2f);   //  returns NULL if not found or if the file is in the root ( / )
  if (file_part == NULL) return NULL;
  return strdup(file_part + 1);
}

//  Extracts file path starting with "/" and ending with "/"
char* extract_file_path(const char *APath)
{
  if (APath == NULL) return NULL;
//  fprintf(stderr, "xxx = %s\n", (APath + strlen(APath) - 1));
  char *file_part = rindex(APath, 0x2f);   //  returns NULL if not found or if the file is in the root ( / )
  if (file_part == NULL) return NULL;
  char *ANewPath = (char*)malloc(file_part - APath + 2);
  snprintf(ANewPath, file_part - APath + 2, "%s", APath);
  return ANewPath;
}
